*THIS PROGRAM produces SCHD_work.dta and related datasets used for analysis
*Prior to running this program, need to run code to produce reshaped_full_student_teacher_data_89_to_09_final.dta, which was run on Columbia servers

global date 040913
global datadir /* path to data folder */

clear all
set more off
set mem 31g
set matsize 5000

use $datadir/reshaped_full_student_teacher_data_89_to_09_final.dta, clear


***DROPPING OBSERVATIONS: ALL LAGS AND LEADS ARE DEFINED BEFORE THIS POINT

g byte grade = student_grade
replace grade = test_grade if grade==.
g int sch = school_number
replace sch = test_school_number if sch==.

*Drop high school tests for which we have no teacher data and students for whom we have no grade info
drop if test_grade==9| test_grade==10

* MAX GRADE LEVEL IN THE SCHOOL-YEAR CELL
        bysort sch year :egen maxgr=max(grade )

* MIN GRADE LEVEL IN THE SCHOOL-YEAR CELL
        bysort sch year :egen mingr=min(grade )

* REPLACE MINIMUM GRADE WITH KG IF SCHOOL RELIES ON TEST SCORES AND HAS MIN GRADE 2 OR 3
        bysort sch year :egen emingr=min(student_grade)
        replace mingr=0 if emingr==. & ((inrange(year,1991,1994) & mingr==2) | (year>1994 & mingr==3))
        drop emingr

* ADD VARIABLE FOR STUDENT MATCHABLE TO IRS DATA, created in irs_match.do
        merge m:1 student_id using $datadir/matchable.dta
        drop if _m==2
        tab _merge
        drop _merge

* RENAME VARIABLES
        rename student_id stuid
        rename z_score score
        rename other_z_score other
        rename l_z_score l_score
        rename l_other_z_score l_other
        rename l2_z_score l2_score
        rename f_z_score f_score
        rename f2_z_score f2_score
        rename student_special sped
        rename student_ethnicity ethnicity
        rename student_female female
        rename student_limited_english limited_english
        rename teacher_id7 teacher

* INDICATOR FOR MATH AND DROP EXTRA VARS
        g byte math=subject=="math"
        drop subject student_zipcode student_geo_code student_home_lang *percentile*

* REPEAT GRADE DUMMY -- USE REGULAR GRADE VARIABLE AND TEST GRADE -- NOTE THAT IN 1991 WE HAVE TO RELY ON TEST GRADE SINCE WE HAVE NO 1990 REGULAR GRADE, AND WE HAVE TO BASE OFF OF 1 OB PER STUDENT-SUBJECT-YEAR
        bysort observation_to_keep math stuid (year): gen p_grade=grade[_n-1] if stuid==stuid[_n-1] & observation_to_keep==1
        bysort stuid math year: egen byte prior_grade=max(p_grade)
        drop p_grade
        g byte repeat_grade = cond(year==1991,0,cond(prior_grade~=. & grade~=.,cond(grade==prior_grade,1,0),.))
        g byte repeat_test  = test_grade==prior_test_grade & prior_test_grade~=. & test_grade~=.

* GENERATE VARIABLE FOR STAYING BACK NEXT YEAR
        bysort observation_to_keep math stuid (year): gen f_tgrade=test_grade[_n+1] if stuid==stuid[_n+1] & observation_to_keep==1
        bysort stuid math year: egen byte future_tgrade=max(f_tgrade)
        drop f_tgrade
        g byte future_repeat = test_grade==future_tgrade if test_grade~=. & future_tgrade~=.


* NO MORE NEED FOR LAGS, SO WE CAN DROP STUDENTS IN YEAR-GRADE CELLS  KG, 1 AND 2
* LATER WE WILL ALSO SET PRIOR CLASS AND SCHOOL SCORES TO MISSING FOR GRADE 3 B/C GRADE 2 PRIOR SCORES ARE TOO COARSE AND INCOMPLETE
        drop if grade<3

***GENERATE COVARIATES FOR VALUE ADDED REGRESSION

* DUMMY FOR HAVING MISSING PRIOR SCORE
        gen MPREO=l_other==.
        replace l_other=0 if MPREO==1

* SET PRIOR OTHER SCORE AS MISSING FOR 8TH GRADERS IN MATH IN 2003--THIS VARIABLE IS GENERALLY MISSING BECAUSE WE HAVE NO 7TH GRADE READING SCORES
* IN 2002.  WE ARE GOING TO DROP THEM FROM THE REGRESSION.
        replace l_other=. if test_grade==8 & year==2003 & math==1

* REPLACE LAGGED OWN SUBJECT SCORE WITH MISSING FOR STUDENTS REPEATING 8TH GRADE IN 2003--THEY'LL HAVE IT NON-MISSING BUT THEIR PEERS (RISING 7TH GRADERS)
* WILL ALL HAVE MISSING VALUES BECAUSE OF NO 7TH GRADE READING TEST SCORES IN 2002
        replace l_score=. if test_grade==8 & year==2003 & math==0

* CUBIC IN PRIOR SCORES
        foreach vary in score other {
                g l_`vary'2 = l_`vary'^2
                g l_`vary'3 = l_`vary'^3
        }

* INTERACT CUBIC IN PRIOR OWN-SUBJECT SCORE WITH INDICATOR FOR MISSING PRIOR OTHER SCORE
        foreach vary of varlist l_score* {
                gen MPREO`vary'=MPREO*`vary'
                }
                renpfix MPREOl MPREOPREZl


* CREATE INTERACTIONS OF PRIOR TEST SCORES WITH GRADE LEVEL
        char test_grade[omit] 4
        xi i.test_grade|l_score i.test_grade|l_score2 i.test_grade|l_score3, prefix(PREZ)
        xi i.test_grade|l_other i.test_grade|l_other2 i.test_grade|l_other3, prefix(PREO)
        drop PREZtest* PREOtest* PREZ*3 PREO*3
         foreach vary of varlist PRE* {
                replace `vary'=. if test_grade==3
                }

* FREE LUNCH INDICATOR
        g byte lunch = (student_mealcode ==1 | student_mealcode ==2 | student_mealcode ==5)
        g byte miss_lunch = student_mealcode==. & year>2003
        drop student_mealcode

* LAGGED ABSENCES AND FLAG FOR PRIOR SUSPENSION
    replace prior_absences=0 if year<=1999
    replace prior_suspensions=cond(year<=1999,0,cond(prior_suspensions>0 & prior_suspensions>0 <.,1,prior_suspensions>0))
    rename prior_absences absences
    rename prior_suspensions suspensions
    drop student_days_absent student_suspensions

* INTERACT SPECIAL ED AND LEP WITH 1999+
        g byte post99_sped = (year>=1999)*sped
        g byte post99_limited_eng = (year>=1999)*limited_eng

* AGE VARIABLES
        g age_current =  (mdy(6,30,year)-mdy(student_dobmm,student_dobdd,student_dobyy))/365
        g age09 =  (mdy(12,31,2009)-mdy(student_dobmm,student_dobdd,student_dobyy))/365
        drop student_dob*

* ETHNICITY DUMMIES
        char ethnicity[omit] 5
        xi, prefix(ETHN) i.ethnicity

* CLASS AND COHORT (RPOOL) GROUPINGS
        egen class = group(math sch year class_id)
        egen rpool = group(math sch year grade)

* CLASS SIZE
        bys class: egen CLass_size=sum(observation_to_keep)

* DEFINE ONE GRADE PER CLASS WHEN WE HAVE CLASS LINKS, ELSE USE REPORTED TEST_GRADE
* WE'RE GOING TO INTERACT WITH THIS INSTEAD OF STUDENTS' INDIVIDUAL TEST GRADE IN ORDER TO HAVE THE SAME VALUE ACROSS STUDENTS WITHIN A CLASSROOM.
        bys class: egen modal_grade = mode(grade) if class~=., max

* NUMBER OF KIDS TAUGHT BY THE TEACHER;
        bys teacher year math modal_grade: egen n_students = sum(observation_to_keep)

* FIX ERRONEOUS CLASS LINKS WITH VERY LARGE "CLASS" SIZES
        replace class = . if CLass_size>50
        replace class_id = "" if CLass_size>50
        replace teacher = . if CLass_size>50 | n_students>200
        drop n_students


* LIST OF CONTROLS--WE'LL NEXT GENERATE CLASS AND SCHOOL-YEAR OR SCHOOL-GRADE LEVEL MEANS
        global listcontrols ETHN* sped female age_current miss_lunch lunch limited_english repeat_grade repeat_test absences suspensions post99_sped post99_limited_eng MPREO


* CLASS LEVEL AVERAGE OF STUDENT-LEVEL CONTROLS, ****WEIGHTING BY STUDENTS' DAYS IN THE CLASS****
        foreach vary of varlist l_score l_other $listcontrols MPREOPREZ* {
                if "`vary'"~="l_other"{
                   bys class: egen CL`vary'=wtmean(`vary'), weight(class_days_enrolled/(end-start))
                       }
                if "`vary'"=="l_other"{
                   bys class: egen CL`vary'=wtmean(cond(MPREO==1,.,`vary')), weight(class_days_enrolled/(end-start))
               }
        }

* SCHOOL-YEAR-SUBJECT LEVEL AVERAGE OF STUDENT-LEVEL CHARACTERISTICS **OTHER THAN PRIOR SCORES**, ****WEIGHTING BY STUDENTS' DAYS IN THE SCHOOL****
        foreach vary of varlist $listcontrols {
           bys sch year math: egen SY`vary'=wtmean(`vary'), weight(school_days_enrolled/(end-start))
           }

*SCHOOL-YEAR-SUBJECT LEVEL AVERAGE OF PRIOR TEST SCORES, ****WEIGHTING BY STUDENTS' DAYS IN THE SCHOOL****
   bys sch modal_grade math: egen SGl_score=wtmean(l_score), weight(school_days_enrolled/(end-start))
   bys sch modal_grade math: egen SGl_other=wtmean(cond(MPREO==1,.,l_other)), weight(school_days_enrolled/(end-start))
  * OPTION TO CALCULATE THIS ONLY FOR STUDENTS WHO DID NOT STAY BACK THE FOLLOWING YEAR--TECHNICALLY THERE IS MECHANICAL CORRELATION WITH CURRENT SCORE
  * bys sch modal_grade math: egen SGl_score=wtmean(cond(future_repeat==1,.,l_score)), weight(school_days_enrolled/(end-start))
  * bys sch modal_grade math: egen SGl_other=wtmean(cond(future_repeat==1,.,cond(MPREO==1,.,l_other))), weight(school_days_enrolled/(end-start))


* SQUARE AND CUBE FOR PRIOR SCORE CLASS AND SCHOOL-GRADE MEANS
        foreach vary in l_score l_other {
        g CL`vary'2 = CL`vary'^2
        g CL`vary'3 = CL`vary'^3
        g SG`vary'2 = SG`vary'^2
        g SG`vary'3 = SG`vary'^3
        }

* CREATE INTERACTIONS OF CLASS AVG PRIOR TEST SCORES WITH GRADE LEVEL
        xi i.modal_grade|CLl_score i.modal_grade|CLl_score2 i.modal_grade|CLl_score3, prefix(CLPZ)
        xi i.modal_grade|CLl_other i.modal_grade|CLl_other2 i.modal_grade|CLl_other3, prefix(CLPO)
        drop CLPZmoda* CLPOmoda*
        capture drop CLPZ*3 CLPO*3
        capture drop CLPZ*0 CLPO*0
        capture drop CLPZ*1 CLPO*1
        capture drop CLPZ*2 CLPO*2
        foreach vary of varlist CLP* {
                replace `vary'=. if modal_grade==3
                }

* CREATE INTERACTIONS OF SCHOOL-GRADE AVG PRIOR TEST SCORES WITH GRADE LEVEL
        xi i.modal_grade|SGl_score i.modal_grade|SGl_score2 i.modal_grade|SGl_score3, prefix(SGPZ)
        xi i.modal_grade|SGl_other i.modal_grade|SGl_other2 i.modal_grade|SGl_other3, prefix(SGPO)
        drop SGPZmoda* SGPOmoda*
        capture drop SGPZ*3 SGPO*3
        capture drop SGPZ*0 SGPO*0
        capture drop SGPZ*1 SGPO*1
        capture drop SGPZ*2 SGPO*2
            foreach vary of varlist SGP* {
                replace `vary'=. if modal_grade==3
                }

* RENAME PRIOR SCORES TO HAVE "PRE" PREFIX, CLASS PRIOR SCORES HAVE CLP AND SCHOOL GRADE PRIOR SCORES HAVE SGP
         for any score other \ any Z O : rename l_X PREYl_X \ rename l_X2 PREYl_X2 \ rename l_X3 PREYl_X3 \ rename CLl_X CLPYl_X \ rename CLl_X2 CLPYl_X2 \ rename CLl_X3 CLPYl_X3
         rename SGl_score SGPZl_score
         rename SGl_score2 SGPZl_score2
         rename SGl_score3 SGPZl_score3
         rename SGl_other SGPOl_other
         rename SGl_other2 SGPOl_other2
         rename SGl_other3 SGPOl_other3

* FLAG HIGH SPEC ED CLASSES
        g byte flag_sped= CLsped>.25 & CLsped<.

* USE CONSIDATED INFO FROM ENROLLMENT AND SERVICE HISTORY FILES TO CHOOSE ONE SCHOOL PER TEACHER PER YEAR;
        merge m:1 teacher sch year using $datadir/final_teacher_list_in_schd_work_schools.dta
        drop if _m==2
        replace teacher=. if _m==1
        drop _merge

* VARIABLE FOR SAMPLE TO USE IN ANALYSIS
        g byte sample_to_keep= observation_to_keep==1 & test_district_id~=75 & district_id~=75 & flag_sped==0 & CLass_size>9.5
        keep if sample_to_keep==1
	
********************************
* ADDED CODE TO DROP SPECIAL ED CLASSROOMS IN MIDDLE SCHOOLS
egen altclass = group(math sch year homeroom_id)
* ONLY USE THIS VARIABLE IF THE MAJORITY OF PEOPLE IN THE HOMEROOM DO NOT HAVE COURSE INFO
bysort altclass: egen mean_missclass=mean(cond(class==.,1,0))
replace altclass=. if mean_missclass<.5
bysort altclass: egen altCLsped=mean(sped)
gen altflag_sped = altCLsped>.25 & altCLsped<. & altclass~=. & class==.
drop if altflag_sped==1
drop altflag_sped altclass altCLsped mean_missclass

***************************
* ADDED CODE TO CREATE MIDDLE "SCHOOL" INDICATOR , I.E. WHETHER THE SCHOOL-GRADE CELL IS USING COURSE CODES AS OPPOSED TO CLASS CODES IN IDENTIFYING TEACHERS
*****************************
cap drop temp
g temp = inlist(substr(class_id,1,1),"M","E")==1
bys sch grade year: egen middle = max(temp)
drop temp

* NEW CLASS SIZE VARIABLE TO CHECK WHETHER WE GOT THE IRS DATA RIGHT BY VERIFYING WHETHER THE COUNT OF PEOPLE IN THE IRS DATA IN EACH CLASS
* EXACTLY MATCHES THE CLASS_SIZE VARIABLE.  SO IT IS IMPORTANT TO GET IT EXACTLY RIGHT IN THE FINAL DATASET.
        rename CLass_size orig_class_size
        bys class: egen CLass_size = sum(sample_to_keep)

        keep if CLass_size>9.5

compress

***************************************************************
************SAMPLE COUNTS REPORTED IN THE PAPER****************
***************************************************************

*15.7 million obs total in grades 4-8
count if grade>3&grade<=8
*9.1 million obs total in grades 4-8 with non-missing teacher
count if grade>3&grade<=8&teacher~=.
*7.7 million obs total in grades 4-8 with non-missing teacher and score gains
count if grade>3&grade<=8&teacher~=.&score~=.&PREZl_score~=.

save $datadir/schd_work_`date', replace

* GENERATE TEACHER-CLASS LIST and other MISC datasets used in analysis program

preserve
keep teacher
keep if teacher~=.
bysort teacher: keep if _n==1
merge 1:m teacher using $datadir/final_teacher_list_in_schd_work_schools.dta
drop if _m==2
drop _m
save $datadir/final_teacher_list_linked_to_students.dta , replace
restore

preserve
keep teacher class_id sch year modal_grade math
keep if teacher~=.
bysort sch year class_id math: keep if _n==1
sort sch year class_id math
save $datadir/teacher_class_list_from_schd_work5, replace
restore

preserve
keep if uniform()<.01
save $datadir/schd_work`date'_1pct, replace
restore

preserve
keep year grade math sch
drop if sch==.|grade==.
bys sch year grade math: g n_rpool = _N
bys sch year grade math: keep if _n==1
sort sch year grade math
save $datadir/sch_grade_enrollments.dta, replace
restore

keep teacher year grade math
keep if grade~=. & teacher~=.
bys teacher year math grade: g n_students = _N
bys teacher year math grade: keep if _n==1
sort teacher year grade math
save $datadir/teacher_enrollments, replace

*EXPORT DATA TO TAKE TO IRS
do irs/irs_export_final.do
